unit hiContPlug;

interface

uses KOL, hxContPlug, Share, Windows, Debug;

type
  THiContPlug = class(TContPlug)
   private
     FFieldIndex, FUnitIndex, FResult: integer;
     FData: TData;

     procedure SetFields(const Value: string);
   public
     procedure Load;
     procedure Unload;
   public
  
    _event_onLoadPlugin: THI_Event;
    _event_onUnloadPlugin: THI_Event;
    _event_onGetValue: THI_Event;
    _event_onLoadFile: THI_Event;
    
    _prop_DetectString: string;

    procedure GetValue(FieldIndex, UnitIndex, Flags, MaxLen: integer; FieldValue: PChar; var Result: integer); override;
    constructor Create; reintroduce;
    procedure OpenFileRead(FN: PChar); override;
    
    procedure _work_doResult(var _Data: TData; Index: word);
    procedure _work_doResultEmpty(var _Data: TData; Index: word);
    procedure _work_doResultError(var _Data: TData; Index: word);

    procedure _var_FileName(var _Data: TData; Index: word);
    procedure _var_FieldIndex(var _Data: TData; Index: word);
    procedure _var_UnitIndex(var _Data: TData; Index: word);
    procedure _var_MaxLen(var _Data: TData; Index: word);
    procedure _var_FieldValue(var _data: TData; Index: word);
    
    property _prop_Fields: string write SetFields;
  end;

implementation

//----------------------------------------------------------
//[CLASS: THiContPlug

constructor THiContPlug.Create;
begin
  inherited Create;
  InitAdd(Load);
end; { of constructor THiContPlug.Create
-----------------------------------------------------------}

procedure THiContPlug.Load;
begin
  _hi_OnEvent(_event_onLoadPlugin);  
end; { of procedure THiContPlug.Load
-----------------------------------------------------------}

procedure THiContPlug.Unload;
begin
  _hi_OnEvent(_event_onUnloadPlugin);
end; { of procedure THiContPlug.Unload
-----------------------------------------------------------} 

procedure THiContPlug._var_FileName;
begin
  dtString(_Data, ReadFileName);
end; { of procedure THiContPlug._var_FileName
-----------------------------------------------------------}

procedure THiContPlug._var_FieldIndex;
begin
  dtInteger(_Data, FFieldIndex);
end; { of procedure THiContPlug._var_FieldIndex
-----------------------------------------------------------}

procedure THiContPlug._var_UnitIndex;
begin
  dtInteger(_Data, FUnitIndex);
end; { of procedure THiContPlug._var_UnitIndex
-----------------------------------------------------------}

procedure THiContPlug._var_MaxLen;
begin
  dtInteger(_Data, FMaxLen);
end; { of procedure THiContPlug._var_MaxLen
-----------------------------------------------------------}

procedure THiContPlug._var_FieldValue;
begin
  dtData(_Data, FData);
end; { of procedure THiContPlug._var_FieldValue
-----------------------------------------------------------}

procedure THiContPlug._work_doResult;
begin
  dtData(FData, _Data);
  FResult := PField(Fields.Items[FFieldIndex]).fdFieldType;
end; { of procedure THiContPlug._work_doResultSet
-----------------------------------------------------------}

procedure THiContPlug._work_doResultEmpty;
begin
  FResult := FT_FIELDEMPTY;
end; { of procedure THiContPlug._work_doResultEmpty
-----------------------------------------------------------}

procedure THiContPlug._work_doResultError;
begin
  FResult := FT_FILEERROR;
end; { of procedure THiContPlug._work_doResultError
-----------------------------------------------------------}

procedure THiContPlug.OpenFileRead;
begin
  _hi_OnEvent(_event_onLoadFile, FN);
end; { of procedure THiContPlug.OpenFileRead
-----------------------------------------------------------}

procedure THiContPlug.GetValue;
begin
  dtNull(FData);
  FResult := FT_FIELDEMPTY;
  FFieldIndex := FieldIndex;
  FUnitIndex := UnitIndex;
  FMaxLen := MaxLen;
  _hi_OnEvent(_event_onGetValue, FieldIndex);
  Result := FResult;
  case Result of
    FT_NUMERIC_32: SetFieldValueInt32(FieldValue, ToInteger(FData)); 
    FT_NUMERIC_FLOATING: SetFieldValueDouble(FieldValue, ToReal(FData));
    FT_BOOLEAN: SetFieldValueBool(FieldValue, Boolean(ToInteger(FData)));
    FT_STRING, FT_MULTIPLECHOICE: SetFieldValueStr(FieldValue, PChar(ToString(FData)), MaxLen);
  end;  
end; { of procedure THiContPlug.GetValue
-----------------------------------------------------------}

function _GetTok(var s:string; const c:char):string;
var p:integer;
begin
   p := Pos(c,s);
   if p > 0 then
     begin
       Result := Copy(s,1,p-1);
       if p = Length(s) then s := ''
       else Delete(s,1,p);
     end
   else
     begin
       Result := s;
       s := '';
     end;
end;

procedure THiContPlug.SetFields;
var
  List: PStrList;
  i, FieldType, Flags: integer;
  s, n, t: string;
begin
  List := NewStrList;
  List.Text := Value;
  for i := 0 to List.Count - 1 do
    begin
      Flags := 0;
      s := List.Items[i];
      if s = '' then continue;
      n := _GetTok(s, ',');
      if n = '' then continue;
      t := LowerCase(_GetTok(s, ','));
      if (t = 'int') or (t = 'int32') then FieldType := FT_NUMERIC_32 else
      if (t = 'real') then FieldType := FT_NUMERIC_FLOATING else
      if (t = 'bool') then FieldType := FT_BOOLEAN else
//      if (t = 'str') then FieldType := FT_STRING else
      if (t = 'choice') then FieldType := FT_MULTIPLECHOICE
      else FieldType := FT_STRING;  
      if n[1] = '@' then
        begin
          Delete(n, 1, 1);
          Flags := Flags or CONTFLAGS_EDIT;
        end;
      AddField(n, FieldType, s, Flags);
    end;      
  List.Free; 
end; { of procedure THiContPlug.SetFields
-----------------------------------------------------------}

//[END OF CLASS: THiContPlug
//---------------------------------------------------------

end.
