library make_ContPlug;

uses
  Windows,
  KOL,
  CGTShare in '..\..\CGTShare.pas';

const
  DEF_EXT = '.wdx';
  DETECTPROP = '_prop_DetectString';
  DETECTPROPLEN = Length(DETECTPROP);

type
  PCGrec = ^TCGrec;
  TCGrec = record
    MainForm: string;
    //Vars, Units, IBody, Dead: PStrList;
    Units, IBody:PStrList;
  end;

function GetAppName: string;
begin
  SetLength(result, 1024);
  SetLength(result, GetModuleFileName(HInstance, PChar(@result[ 1 ]), 1024));
end;

function GetClassName(const ElementName: string): string;
var
  i, Len: integer;
begin
  Len := Length(ElementName);
  for i := Len downto 1 do
    if ElementName[i] = '_' then
      begin
        Dec(PInteger(@i)^);
        break;
      end;
  if i <= 1 then
    i := Len;
  Result := 'THi' + Copy(ElementName, 1, i);
end;

function buildGetParamsProc(var params: TBuildParams): integer; cdecl;
begin
  params.flags := CGMP_COMPRESSED or CGMP_RUN or CGMP_RUN_DEBUG;
  Result := CG_SUCCESS;
end;

function buildMakePrj(const params: TBuildMakePrjRec): integer; cdecl;
var
  dt: string;
  usedt: boolean;
  function GetValue(const s: string): string;
  var
    i, k: integer;
  begin
    for i := 1 to Length(s) do if s[i] = '''' then
      for k := Length(s) downto i do if s[k] = '''' then
        begin
          Result := Copy(s, i, k - i + 1);
          exit;
        end;
    Result := '''' + '''';
  end;
  function CheckDetectString(List: PStrList): boolean;
  var
    i, p: integer;
  begin
    for i := 0 to List.Count - 1 do
      begin
        p := PosEx(':', List.Items[i], DETECTPROPLEN);
        if p > 0 then
          if StrComp_NoCase(PChar(DETECTPROP), PChar(Trim(Copy(List.Items[i], 1, p - 1)))) = 0 then
            begin
              dt := GetValue(List.Items[i]);
              Result := (dt > '''''');
              exit;
            end;
      end;
    Result := false;
  end;
var
  R: PStrList;
  i: integer;
  P: PCGrec;
begin
  p := params.result;
  dt := '';
  R := NewStrList;
  R.Add('library HiAsm;');
  R.Add('');
  R.Add('uses');
  for i := 0 to p.Units.Count - 1 do
    R.Add('  ' + p.Units.Items[i] + ',');
  R.Add('  KOL, Share, hxContPlug, hiContPlug;');
  R.Add('');
  R.Add('var ClassProject: ' + GetClassName(p.MainForm) + ';');
  R.Add('');
  usedt := CheckDetectString(p.IBody);
  if usedt then
    begin
      R.Add('procedure ContentGetDetectString(DetectString: PChar; MaxLen: integer); stdcall;');
      R.Add('begin');
      R.Add('  _strcpy(DetectString, ' + GetValue(dt) + ', MaxLen);');
      R.Add('end;');
    end;
  R.Add('');
  R.Add('procedure ContentSetDefaultParams(DPS: PContentDefaultParamStruct); stdcall;');
  R.Add('begin');
  R.Add('  ClassMain := TClass' + p.MainForm + '.Create;');
  R.Add('  ClassProject := ClassMain.' + p.MainForm + ';');
  //R.Add('  ClassProject.Load();');
  R.Add('  EventOn;');
  R.Add('  InitDo;');
  R.Add('end;');
  R.Add('');
  R.Add('procedure ContentPluginUnloading; stdcall;');
  R.Add('begin');
  R.Add('  ClassProject.Unload();');
  R.Add('  ClassMain.Free;');
  R.Add('end;');
  R.Add('');
  R.Add('exports');
  if usedt then
    R.Add(  'ContentGetDetectString,');
  R.Add('  ContentSetDefaultParams,');
  R.Add('  ContentPluginUnloading;');
  R.Add('');
  R.Add('{$R allres.res}');
  R.Add('');
  R.Add('end.');

  R.SaveToFile(params.prjFilename);
  R.free;

  Result := CG_SUCCESS;
end;

function buildCompliteProc(const params: TBuildCompliteRec): integer; cdecl;
var
  src: string;
begin
  src := ExtractFilePath(params.prjFilename) + ExtractFileNameWOext(params.prjFilename) + '.dll';
  MoveFileEx(PChar(src), PChar(params.appFilename), MOVEFILE_COPY_ALLOWED or MOVEFILE_REPLACE_EXISTING);
  Result := CG_SUCCESS;
end;

function buildRunProc(var params: TBuildRunRec): integer; cdecl;
var
  tcpath, tcparams: string;
  procedure ReadIni;
  var
    ini: string;
  begin
    
    with OpenIniFile(ChangeFileExt(GetAppName, '.ini'))^ do
      begin
        Section := 'make';
        tcpath := ValueString('Path', '');
        if not FileExists(tcpath) then
          tcpath := '%programfiles%\Total Commander\TOTAL.EXE';
        ini := ValueString('IniName', '');
        tcparams := ValueString('Params', '');
        Free;
        if ini <> '' then
          begin
            ini := '/i=' + ini;
            if Pos(' ', ini) > 0 then ini := '"' + ini + '"';
          end;
        if (tcparams > '') and (ini > '') then tcparams := ' ' + tcparams;
        tcparams := ini + tcparams;
      end;

  end;
var
  si: TStartupInfo;
  p: TProcessInformation;
  res: cardinal;
begin
  FillChar(Si, SizeOf(Si), 0);
  with Si do
    begin
      cb := SizeOf(Si);
      dwFlags := STARTF_USESHOWWINDOW;
      wShowWindow := 4;
    end;
  ReadIni;
  CreateProcess(nil, PChar('"' + tcpath + '" ' + tcparams), nil, nil, false, CREATE_DEFAULT_ERROR_MODE, nil, nil, si, p);
  res := WAIT_TIMEOUT;
  Params.data := pointer(p.hProcess);
  while (res = WAIT_TIMEOUT) do
    res := WaitForSingleObject(p.hProcess, 10);

  Result := CG_SUCCESS;
end;

function buildStopProc(var params: TBuildRunRec): integer; cdecl;
begin
  TerminateProcess(cardinal(Params.data), 0);
  Result := CG_SUCCESS;
end;

exports
  buildGetParamsProc,
  buildMakePrj,
  buildCompliteProc,
  buildRunProc,
  buildStopProc;

begin


end.

